package statemachinetest.dao;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;

import statemachinetest.service.IStateMachineTestConstants;

public class ExcelInformationWriter {

	private String fileName;

	private Workbook workbook;

	private List<List<String>> stateEventMetrixDataByRows;

	private List<List<String>> stateStateMetrixDataByRows;
	
	private List<List<String>> switchCoverageTestCaseRows;

	/**
	 * @param stateEventMetrixDataByRows
	 *            o͂eicState x Eventj
	 * @param stateStateMetrixDataByRows
	 *            o͂eicState x Statej
	 * @param fileName
	 *            o͂t@CpX
	 */
	public ExcelInformationWriter(
			List<List<String>> stateEventMetrixDataByRows,
			List<List<String>> stateStateMetrixDataByRows, 
			List<List<String>> switchCoverageTestCaseRows, 
			String fileName) {

		this.stateEventMetrixDataByRows = stateEventMetrixDataByRows;
		this.stateStateMetrixDataByRows = stateStateMetrixDataByRows;
		this.switchCoverageTestCaseRows = switchCoverageTestCaseRows;
		
		this.fileName = fileName;
		this.workbook = new HSSFWorkbook();
	}

	private void setContetsToCells() throws IOException {
		Sheet stateEventSheet = workbook.createSheet();
		Sheet stateStateSheet = workbook.createSheet();
		Sheet switchCoverageSheet = workbook.createSheet();

		workbook.setSheetName(0, IStateMachineTestConstants.STATE_EVENT);
		workbook.setSheetName(1, IStateMachineTestConstants.STATE_STATE);
		workbook.setSheetName(2, IStateMachineTestConstants.TESTCASE_TRANPATH);

		if(stateEventMetrixDataByRows != null){
			createCellDataFromModeData(stateEventSheet, stateEventMetrixDataByRows);
		}
		if(stateStateMetrixDataByRows != null){
			createCellDataFromModeData(stateStateSheet, stateStateMetrixDataByRows);
		}
		if(switchCoverageTestCaseRows != null){
			createCellDataFromModeData(switchCoverageSheet, switchCoverageTestCaseRows);
		}

	}


	
	private void createCellDataFromModeData(Sheet sheet,
			List<List<String>> metricsDataByRowList) {

		CellStyle style = workbook.createCellStyle();
		CellStyle columnStyle = workbook.createCellStyle();
		CellStyle rowStyle = workbook.createCellStyle();

		setCellStyle(style);
		setCellStyle(columnStyle);
		setCellStyle(rowStyle);

		int rowCount = 0;
		for (Iterator<List<String>> rows = metricsDataByRowList.iterator(); rows.hasNext(); rowCount++) {
			List<String> line = (List<String>) rows.next();
			Row row = sheet.createRow(rowCount);

			int cellCount = 0;
			for (Iterator cells = line.iterator(); cells.hasNext(); cellCount++) {
				Cell cell = row.createCell(cellCount);
				cell.setCellValue((String) cells.next());

				if (rowCount == 0) {
					columnStyle.setFillForegroundColor(IndexedColors.AQUA
							.getIndex());
					cell.setCellStyle(columnStyle);
				} else if (cellCount == 0) {
					rowStyle.setFillForegroundColor(IndexedColors.LIGHT_YELLOW
							.getIndex());
					cell.setCellStyle(rowStyle);
				} else {
					style
							.setFillForegroundColor(IndexedColors.WHITE
									.getIndex());
					cell.setCellStyle(style);
				}
			}
		}
	}

	private void setCellStyle(CellStyle style) {
		style.setWrapText(true);
		style.setBorderBottom(CellStyle.BORDER_THIN);
		style.setBorderTop(CellStyle.BORDER_THIN);
		style.setBorderLeft(CellStyle.BORDER_THIN);
		style.setBorderRight(CellStyle.BORDER_THIN);
		style.setFillPattern(CellStyle.SOLID_FOREGROUND);
	}

	/**
	 * t@Cɏo͂B
	 * 
	 * @throws IOException
	 */

	public void createExcelFile() throws IOException {

		FileOutputStream outStream = null;

		try {
			setContetsToCells();

			outStream = new FileOutputStream(fileName);
			workbook.write(outStream);

		} finally {
			if (outStream != null) {
				outStream.close();
			}
		}
	}
}
