package statemachinetest.service;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;

import statemachinetest.service.builder.TestCaseOfStateTranPathBuilder;

import com.change_vision.jude.api.inf.model.IClass;
import com.change_vision.jude.api.inf.model.IDiagram;
import com.change_vision.jude.api.inf.model.IElement;
import com.change_vision.jude.api.inf.model.IModel;
import com.change_vision.jude.api.inf.model.INamedElement;
import com.change_vision.jude.api.inf.model.IOperation;
import com.change_vision.jude.api.inf.model.IPackage;
import com.change_vision.jude.api.inf.model.IPseudostate;
import com.change_vision.jude.api.inf.model.IState;
import com.change_vision.jude.api.inf.model.IStateMachine;
import com.change_vision.jude.api.inf.model.IStateMachineDiagram;
import com.change_vision.jude.api.inf.model.ITransition;
import com.change_vision.jude.api.inf.model.IVertex;

/**
 * w肳ꂽvWFNgAԑJڕ\\zNXB
 */
public class StateMachineTestBuilder implements IStateMachineTestConstants {
	/*
	 * key :dgm [IStateMachineDiagram] value:dgm's parent [INamedElement]
	 */
	private Map<IDiagram, INamedElement> stateMachineDgmParentMap = new HashMap<IDiagram, INamedElement>();
	/*
	 * key :state [IState] value:state's parent state[IState]
	 */
	private Map<IVertex, IVertex> stateParentMap = new HashMap<IVertex, IVertex>();

	public StateMachineTestBuilder() {
		// set up information
		stateMachineDgmParentMap.clear();

	}

	protected IStateMachineDiagram[] getAllStateMachineDiagrams(IModel iModel) {

		List<IDiagram> diagrams = getAllStateMachineDiagrams(iModel,
				new ArrayList<IDiagram>());
		IStateMachineDiagram[] diagramArrays = (IStateMachineDiagram[]) diagrams
				.toArray(new IStateMachineDiagram[diagrams.size()]);

		return diagramArrays;

	}

	private List<IDiagram> getAllStateMachineDiagrams(
			INamedElement iNamedElement, List<IDiagram> diagrams) {
		IDiagram[] dgms = iNamedElement.getDiagrams();
		for (int i = 0; i < dgms.length; i++) {
			if (dgms[i] instanceof IStateMachineDiagram) {
				diagrams.add(dgms[i]);
				stateMachineDgmParentMap.put(dgms[i], iNamedElement);
			}
		}
		if (iNamedElement instanceof IPackage) {
			INamedElement[] children = ((IPackage) iNamedElement)
					.getOwnedElements();
			for (int i = 0; i < children.length; i++) {
				diagrams = getAllStateMachineDiagrams(
						(INamedElement) children[i], diagrams);
			}
		}
		if (iNamedElement instanceof IClass) {
			IOperation[] children = ((IClass) iNamedElement).getOperations();
			for (int i = 0; i < children.length; i++) {
				diagrams = getAllStateMachineDiagrams(
						(INamedElement) children[i], diagrams);
			}
		}
		return diagrams;
	}

	protected String getOutputFileName(String outputDir,
			IStateMachineDiagram dgm) {
		String name = getFullPathString(dgm);
		String dir = name;
		if (dir.indexOf("/") != -1) {
			dir = dir.substring(0, dir.lastIndexOf("/"));
			// create Output Folder
			createOutputFolder(outputDir + "/" + dir);
		} else {
			// create Output Folder
			createOutputFolder(outputDir + "/");
		}

		return outputDir + "/" + name + ".xls";
	}

	private String getFullPathString(IStateMachineDiagram dgm) {
		StringBuffer sb = new StringBuffer();
		INamedElement namespace = null;
		IStateMachine iStateMachine = dgm.getStateMachine();
		if (iStateMachine == null) {
			return "";
		}
		IElement elem = (IElement) stateMachineDgmParentMap.get(dgm);
		if (elem instanceof INamedElement) {
			namespace = (INamedElement) elem;
			INamedElement parent = (INamedElement) namespace.getOwner();
			sb.insert(0, dgm.getName());
			if (parent != null) {
				sb.insert(0, "/");
				sb.insert(0, namespace.getName());
			}
			while (parent != null) {
				if (parent.getOwner() != null) {
					sb.insert(0, "/");
					sb.insert(0, parent.getName());
				}
				parent = (INamedElement) parent.getOwner();
			}
		}
		return sb.toString();
	}

	protected void createOutputFolder(String outputDir) {
		File file = new File(outputDir);
		if (!file.exists()) {
			file.mkdirs();
		}
	}

	protected List<List<String>> getContentsRowsList(
			IStateMachineDiagram iStateMachineDiagram, String mode) {

		List<List<String>> contents = new ArrayList<List<String>>();

		// add header infomation
		contents.add(getHeader(iStateMachineDiagram, mode));
		// add rows
		setStateMachineDiagramInfos(iStateMachineDiagram, contents, mode);

		return contents;

	}

	protected List<String> getHeader(IStateMachineDiagram stateMachineDgm,
			String mode) {

		List<String> header = new ArrayList<String>();

		if (mode.equals(MODE_STATE_EVENT)) {
			header.add(HEADER_STATE_EVENT);
			header.addAll(getAllEvents(stateMachineDgm));
		} else {
			header.add(HEADER_STATE_STATE);
			header.addAll(getAllStates(stateMachineDgm));
		}

		return header;
	}

	protected List<String> getAllEvents(IStateMachineDiagram stateMachineDgm) {

		List<String> events = new ArrayList<String>();
		IStateMachine iStateMachine = stateMachineDgm.getStateMachine();

		addAllEventFromTransitions(events, iStateMachine);
		addAllEventFromInternalTransitions(events, iStateMachine);

		return events;
	}

	private void addAllEventFromInternalTransitions(List<String> events,
			IStateMachine iStateMachine) {
		ITransition[] iTransition;
		IVertex[] iVertex = iStateMachine.getVertexes();
		for (int i = 0; i < iVertex.length; i++) {
			if (iVertex[i] instanceof IState) {
				iTransition = ((IState) iVertex[i]).getInternalTransitions();
				for (int j = 0; j < iTransition.length; j++) {
					addEventString(events, iTransition[j].getEvent(),
							iTransition[j].getGuard());
				}
			}
		}
	}

	protected void addAllEventFromTransitions(List<String> events,
			IStateMachine iStateMachine) {

		ITransition[] iTransition = iStateMachine.getTransitions();

		for (int i = 0; i < iTransition.length; i++) {
			addEventString(events, iTransition[i].getEvent(), iTransition[i]
					.getGuard());
		}
	}

	private void addEventString(List<String> events, String event, String guard) {
		if (event != null && event.equals("")) {
			event = EVENT_NOTHING;
		}
		if (event != null && !events.contains(event)) {
			if (guard != null && !guard.equals("")) {
				event += " [" + guard + "]";
			}
			if (!events.contains(event)) {
				events.add(event);
			}
		}
	}

	private List<IVertex> getAllStateList(IStateMachineDiagram stateMachineDgm) {

		List<IVertex> states = new ArrayList<IVertex>();

		IStateMachine iStateMachine = stateMachineDgm.getStateMachine();
		IVertex[] iVertex = iStateMachine.getVertexes();

		// set up information
		setUpStateparentMap(iVertex, null);
		addAllStateList(states, iVertex);

		return states;
	}

	protected List<String> getAllStates(IStateMachineDiagram stateMachineDgm) {

		List<String> states = new ArrayList<String>();

		IStateMachine iStateMachine = stateMachineDgm.getStateMachine();
		IVertex[] iVertex = iStateMachine.getVertexes();

		// set up information
		setUpStateparentMap(iVertex, null);
		addAllStateString(states, iVertex);

		return states;
	}

	private void setUpStateparentMap(IVertex[] iVertexs, IVertex parentVertex) {
		for (int i = 0; i < iVertexs.length; i++) {
			stateParentMap.put(iVertexs[i], parentVertex);
		}
	}

	private String getParentName(IVertex iVertex) {

		StringBuffer sb = new StringBuffer();
		IVertex parentVertex = (IVertex) stateParentMap.get(iVertex);

		while (parentVertex != null) {
			sb.insert(0, "::");
			sb.insert(0, parentVertex.getName());
			parentVertex = (IVertex) stateParentMap.get(parentVertex);
		}

		return sb.toString();
	}

	private void addAllStateList(List<IVertex> states, IVertex[] iVertex) {

		for (int i = 0; i < iVertex.length; i++) {

			if (iVertex[i] instanceof IState) {
				states.add(iVertex[i]);

				IVertex[] isubVertex = ((IState) iVertex[i]).getSubvertexes();

				setUpStateparentMap(isubVertex, (IState) iVertex[i]);
				addAllStateList(states, isubVertex);

			} else if (iVertex[i] instanceof IPseudostate
					&& ((IPseudostate) iVertex[i]).isInitialPseudostate()) {

				states.add(iVertex[i]);

			}
		}
	}

	private void addAllStateString(List<String> states, IVertex[] iVertex) {

		for (int i = 0; i < iVertex.length; i++) {

			if (iVertex[i] instanceof IState) {
				String parentName = getParentName(iVertex[i]);
				states.add(parentName + iVertex[i].getName());

				IVertex[] isubVertex = ((IState) iVertex[i]).getSubvertexes();

				// set up information
				setUpStateparentMap(isubVertex, (IState) iVertex[i]);
				addAllStateString(states, isubVertex);

			} else if (iVertex[i] instanceof IPseudostate
					&& ((IPseudostate) iVertex[i]).isInitialPseudostate()) {

				String parentName = getParentName(iVertex[i]);
				states.add(parentName + iVertex[i].getName());
			}

		}
	}

	private void setStateMachineDiagramInfos(
			IStateMachineDiagram iStateMachineDiagram,
			List<List<String>> contents, String mode) {

		// State_Event [hłgpĂȂ
		List<String> allEvents = getAllEvents(iStateMachineDiagram);

		stateParentMap.clear();

		// Ԃ܂ޑSĂ̏Ԃ(Xg)Ŏ擾
		List<IVertex> allStates = getAllStateList(iStateMachineDiagram);

		for (int i = 0; i < allStates.size(); i++) {

			List<String> statesNames = new ArrayList<String>();
			IVertex iVertex = (IVertex) allStates.get(i);

			String parentName = getParentName(iVertex);
			statesNames.add(parentName + iVertex.getName());

			if (mode.equals(MODE_STATE_EVENT)) {
				addActionAndState((IVertex) allStates.get(i), allEvents,
						statesNames);

			} else {

				if (mode.equals(MODE_STATE_STATE)) {
					addActionAndEvent((IVertex) allStates.get(i), allStates,
							statesNames);
				}
			}

			contents.add(statesNames);
		}
	}

	// STATE_EVENT mode ̎Ă΂Ȃ
	// JgXe[g̑JڏiCxgAANVAJڐXe[gjXgɓ
	private void addActionAndState(IVertex currentState,
			List<String> allEvents, List<String> allStatesNames) {

		// JgXe[gJڏ擾
		ITransition[] outgoingTransitions = currentState.getOutgoings();

		for (int i = 0; i < allEvents.size(); i++) {

			String eventName = (String) allEvents.get(i);

			boolean isAdd = addActionAndStateForTransitions(currentState,
					allStatesNames, outgoingTransitions, eventName);

			// for internal initial state
			if (!isAdd && eventName.equals(EVENT_NOTHING)) {
				isAdd = addActionAndStateForInternalInitialState(currentState,
						allStatesNames, isAdd);
			}
			if (!isAdd) {
				allStatesNames.add(FORBIDDEN);
			}
		}
	}

	// Event쐬 [Guard]Event
	private String getRefEventName(ITransition outgoingTransition) {

		String refEvent = outgoingTransition.getEvent();
		String refGuard = outgoingTransition.getGuard();

		if (refEvent != null && refEvent.equals("")) {
			refEvent = EVENT_NOTHING;
		}

		if (refGuard != null && !refGuard.equals("")) {
			refEvent += " [" + refGuard + "]";
		}

		return refEvent;
	}

	private boolean addActionAndStateForTransitions(IVertex currentState,
			List<String> allStateNames, ITransition[] outgoingTransitions,
			String eventName) {

		boolean isAdd = false;

		for (int j = 0; j < outgoingTransitions.length; j++) {

			ITransition outgoingTransition = outgoingTransitions[j];

			// Cxg擾
			String refEvent = getRefEventName(outgoingTransition);

			// ̏Ԏ擾
			IVertex nextState = outgoingTransition.getTarget();

			StringBuffer actionStateStr = new StringBuffer();
			// nextState:JunctionPseudostate
			if (isJunctionPseudostate(nextState)) {
				// pass
				// nextState:ChoicePseudostate
			} else if (isChoicePseudostate(nextState)) {
				addActionAndStateForNextChoice(eventName, outgoingTransition,
						refEvent, nextState, actionStateStr);
				// nextState:ForkPseudostate
			} else if (isForkPseudostate(nextState)) {
				addActionAndStateForNextFork(eventName, refEvent, nextState,
						actionStateStr);
				// nextState:JoinPseudostate
			} else if (isJoinPseudostate(nextState)) {
				// pass
				// nextState:others
			} else {
				if (eventName.equals(refEvent)) {
					actionStateStr.append(outgoingTransition.getAction());
					actionStateStr.append(SEPARATOR);
					actionStateStr.append(getParentName(currentState));
					actionStateStr.append(nextState.getName());
				}
			}
			if (actionStateStr.length() != 0) {
				allStateNames.add(actionStateStr.toString());
				isAdd = true;
			}
		}
		return isAdd;
	}

	private void addActionAndStateForNextChoice(String event,
			ITransition outgoingTransition, String refEvent, IVertex nextState,
			StringBuffer actionStateStr) {

		ITransition[] choiceTransitions = nextState.getOutgoings();
		String[] choiceGuards = new String[choiceTransitions.length];

		for (int k = 0; k < choiceTransitions.length; k++) {

			IVertex choiceNextState = choiceTransitions[k].getTarget();
			choiceGuards[k] = choiceTransitions[k].getGuard();

			if (event.equals(refEvent)) {

				String trasitionalInfo = getTransitionInfo(
						getParentName(choiceNextState)
								+ choiceNextState.getName(), "",
						outgoingTransition.getAction());
				if (!StringUtils.isEmpty(choiceGuards[k])) {
					actionStateStr.append(trasitionalInfo + CONTINUOUS
							+ choiceGuards[k]);
				}
			}
		}
	}

	private void addActionAndStateForNextFork(String event, String refEvent,
			IVertex nextState, StringBuffer actionStateStr) {

		ITransition[] choiceTransitions = nextState.getOutgoings();
		String[] choiceGuards = new String[choiceTransitions.length];

		for (int k = 0; k < choiceTransitions.length; k++) {

			ITransition choiceTransition = choiceTransitions[k];
			IVertex choiceNextState = choiceTransition.getTarget();
			choiceGuards[k] = choiceTransitions[k].getGuard();

			if (event.equals(refEvent)) {

				actionStateStr.append(choiceTransition.getAction());
				actionStateStr.append(SEPARATOR);
				actionStateStr.append(getParentName(choiceNextState));
				actionStateStr.append(choiceNextState.getName());
				actionStateStr.append("\n");
			}

		}
	}

	private boolean addActionAndStateForInternalInitialState(
			IVertex currentState, List<String> rows, boolean isAdd) {

		if (currentState instanceof IState) {

			IVertex[] isubVertexs = ((IState) currentState).getSubvertexes();

			for (int j = 0; j < isubVertexs.length; j++) {

				IVertex isubVertex = isubVertexs[j];

				if (isInitialPseudostate(isubVertex)) {
					StringBuffer actionStateStr = new StringBuffer();

					actionStateStr.append(SEPARATOR);
					actionStateStr.append(getParentName(isubVertex));
					actionStateStr.append(isubVertex.getName());

					rows.add(actionStateStr.toString());
					isAdd = true;
					break;
				}
			}
		}
		return isAdd;
	}

	private void addActionAndEvent(IVertex currentState, List<IVertex> states,
			List<String> rows) {

		for (int i = 0; i < states.size(); i++) {

			IVertex nextState = (IVertex) states.get(i);

			StringBuffer actionEventStr = addActionAndEventForTransitions(
					currentState, nextState);

			// for internal initial state
			if (actionEventStr.length() == 0) {
				actionEventStr = addActionAndEventForInternalInitialState(
						currentState, nextState, actionEventStr);
			}
			if (actionEventStr.length() != 0) {
				rows.add(actionEventStr.toString());
			} else {
				rows.add(FORBIDDEN);
			}
		}
	}

	private StringBuffer addActionAndEventForTransitions(IVertex currentState,
			IVertex nextState) {

		ITransition[] outgoingTransitions = currentState.getOutgoings();
		StringBuffer actionEventStr = new StringBuffer();

		for (int j = 0; j < outgoingTransitions.length; j++) {

			IVertex transitionNextState = outgoingTransitions[j].getTarget();
			// transitionNextState:JunctionPseudostate

			if (isJunctionPseudostate(transitionNextState)) {
				// pass
				// transitionNextState:ChoicePseudostate

			} else if (isChoicePseudostate(transitionNextState)) {

				addActionAndEventForNextChoice(nextState, actionEventStr,
						outgoingTransitions[j], transitionNextState);
				// transitionNextState:ForkPseudostate

			} else if (isForkPseudostate(transitionNextState)) {

				addActionAndEventForNextFork(nextState, actionEventStr,
						outgoingTransitions[j], transitionNextState);
				// } else if (isJoinPseudostate(nextState)) {
				// ::TODO
				// transitionNextState:others

			} else {

				if (nextState.equals(transitionNextState)) {
					String event = outgoingTransitions[j].getEvent();
					String guard = outgoingTransitions[j].getGuard();
					String action = outgoingTransitions[j].getAction();

					if (StringUtils.isEmpty(event)
							&& StringUtils.isEmpty(guard)
							&& StringUtils.isEmpty(action)) {

						event = EVENT_NOTHING;

					}

					String trasitionalInfo = getTransitionInfo(event, guard,
							action);

					actionEventStr.append(trasitionalInfo);
					break;
				}
			}
		}
		return actionEventStr;
	}

	private void addActionAndEventForNextChoice(IVertex nextState,
			StringBuffer actionEventStr, ITransition outgoingTransition,
			IVertex transitionNextState) {

		ITransition[] choiceOutgoingTransitions = transitionNextState
				.getOutgoings();

		for (int k = 0; k < choiceOutgoingTransitions.length; k++) {

			ITransition choiceOutgoingTransition = choiceOutgoingTransitions[k];
			IVertex choiceNextState = choiceOutgoingTransition.getTarget();

			if (nextState.equals(choiceNextState)) {
				actionEventStr.append(getTransitionInfo(outgoingTransition
						.getEvent(), choiceOutgoingTransition.getGuard(),
						outgoingTransition.getAction()));
				break;
			}
		}
	}

	protected String getTransitionInfo(String event, String guard, String action) {
		StringBuffer transitionInfoBuffer = new StringBuffer();

		boolean spaceFlg = false;
		if (!StringUtils.isEmpty(event)) {
			transitionInfoBuffer.append(event);
			spaceFlg = true;
		}

		if (!StringUtils.isEmpty(guard)) {
			transitionInfoBuffer.append("[" + guard + "]");
			spaceFlg = true;
		}

		if (!StringUtils.isEmpty(action)) {
			if (spaceFlg) {
				transitionInfoBuffer.append(" " + SEPARATOR + " ");
			} else {
				transitionInfoBuffer.append(" " + SEPARATOR + " ");
			}

			transitionInfoBuffer.append(action);
		}

		return transitionInfoBuffer.toString();
	}

	private void addActionAndEventForNextFork(IVertex nextState,
			StringBuffer actionEventStr, ITransition outgoingTransition,
			IVertex transitionNextState) {

		ITransition[] forkOutgoingTransitions = transitionNextState
				.getOutgoings();

		for (int k = 0; k < forkOutgoingTransitions.length; k++) {

			ITransition forkOutgoingTransition = forkOutgoingTransitions[k];
			IVertex forkNextState = forkOutgoingTransition.getTarget();

			if (nextState.equals(forkNextState)) {

				String trasitionalInfo = getTransitionInfo(outgoingTransition
						.getEvent(), outgoingTransition.getGuard(),
						forkOutgoingTransition.getAction());

				actionEventStr.append(trasitionalInfo);

				break;
			}
		}
	}

	private StringBuffer addActionAndEventForInternalInitialState(
			IVertex currentState, IVertex nextState, StringBuffer actionEventStr) {

		if (currentState instanceof IState) {

			IVertex[] isubVertexs = ((IState) currentState).getSubvertexes();

			for (int j = 0; j < isubVertexs.length; j++) {
				IVertex isubVertex = isubVertexs[j];
				if (nextState.equals(isubVertex)
						&& isInitialPseudostate(isubVertex)) {

					actionEventStr.append(EVENT_NOTHING);

					break;
				}
			}
		}
		return actionEventStr;
	}

	private boolean isChoicePseudostate(IVertex target) {
		if (target instanceof IPseudostate) {
			return ((IPseudostate) target).isChoicePseudostate();
		}
		return false;
	}

	private boolean isJunctionPseudostate(IVertex target) {
		if (target instanceof IPseudostate) {
			return ((IPseudostate) target).isJunctionPseudostate();
		}
		return false;
	}

	private boolean isForkPseudostate(IVertex target) {
		if (target instanceof IPseudostate) {
			return ((IPseudostate) target).isForkPseudostate();
		}
		return false;
	}

	private boolean isJoinPseudostate(IVertex target) {
		if (target instanceof IPseudostate) {
			return ((IPseudostate) target).isJoinPseudostate();
		}
		return false;
	}

	private boolean isInitialPseudostate(IVertex target) {
		if (target instanceof IPseudostate) {
			return ((IPseudostate) target).isInitialPseudostate();
		}
		return false;
	}

	protected void closingProc() {
		stateMachineDgmParentMap.clear();
		stateParentMap.clear();

	}

	protected List<List<String>> getTestCasesByTransitionPath(
			IStateMachineDiagram iStateMachineDiagram, int centerStateCount) {

		TestCaseOfStateTranPathBuilder coverageBuilder = new TestCaseOfStateTranPathBuilder();

		List<List<String>> contents = getContentsRowsList(iStateMachineDiagram,
				MODE_STATE_STATE);

		return coverageBuilder.build(contents, centerStateCount);
	}
}
