package statemachinetest.service;

import java.io.IOException;
import java.util.List;

import statemachinetest.dao.ExcelInformationWriter;

import com.change_vision.jude.api.inf.exception.LicenseNotFoundException;
import com.change_vision.jude.api.inf.exception.NonCompatibleException;
import com.change_vision.jude.api.inf.exception.ProjectLockedException;
import com.change_vision.jude.api.inf.exception.ProjectNotFoundException;
import com.change_vision.jude.api.inf.model.IStateMachineDiagram;
import com.change_vision.jude.api.inf.project.ProjectAccessor;
import com.change_vision.jude.api.inf.project.ProjectAccessorFactory;

public class StateMachineTestBuilderService {


	/**
	 * @param inputFile
	 *            input astah project file
	 *            
	 * @param outputDir
	 *            output folder
	 *            
	 * @param centerStateCount
	 * 			  count of center state 
	 */
	public void export(String inputFile, String outputDir, String centerStateCountString)
			throws LicenseNotFoundException, ProjectNotFoundException,
			NonCompatibleException, IOException, ClassNotFoundException,
			ProjectLockedException, Throwable {

		ProjectAccessor prjAccessor = null;
		StateMachineTestBuilder builder = new StateMachineTestBuilder();

		try {
			prjAccessor = ProjectAccessorFactory.getProjectAccessor();

			// open project
			prjAccessor.open(inputFile, true, false, true);
			
			// get StateMachine Diagrams
			IStateMachineDiagram[] dgms = builder.getAllStateMachineDiagrams(prjAccessor
					.getProject());
			
			int centerStateCount = 0;
			
			try{
			
				centerStateCount = Integer.parseInt(centerStateCountString);

			}catch(NumberFormatException e){
				centerStateCount = 1;
			}
			
			for (int i = 0; i < dgms.length; i++) {

				// create STATE_EVENT data
				List<List<String>> stateEventMetrixDataByRows = builder.getContentsRowsList(
						dgms[i], IStateMachineTestConstants.MODE_STATE_EVENT);

				// create STATE_STATE data
				List<List<String>> stateStateMetrixDataByRows = builder.getContentsRowsList(
						dgms[i], IStateMachineTestConstants.MODE_STATE_STATE);
				
				// create test case data
				List<List<String>> testCases = builder.getTestCasesByTransitionPath(dgms[i], 
						centerStateCount);
				
				
				// createOutputFolder
				builder.createOutputFolder(outputDir);
				
				ExcelInformationWriter writer = new ExcelInformationWriter(
						stateEventMetrixDataByRows, 
						stateStateMetrixDataByRows,
						testCases,
						builder.getOutputFileName(outputDir, dgms[i]));
			
				writer.createExcelFile();

			}
			
		} finally {
			builder.closingProc();
			// close project
			prjAccessor.close();
		}
	}

}
