package statemachinetest.service.builder;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;

import statemachinetest.domain.TransitInfo;
import statemachinetest.service.IStateMachineTestConstants;

public class TestCaseOfStateTranPathBuilder {

	// ^CgƃCfbNX
	private Map<Integer,String> columnTitleMap;;

	// s^CgƃCfbNX
	private Map<Integer,String> rowTitleMap;
		

	public List<List<String>> build(List<List<String>> contents, int centerStateCount) {

		initSetUpTitleData(contents);

		// contentswb_[𔲂Cxg̃Xg
		List<List<TransitInfo>> rowEventList = getRowEventList(contents);

		// Cxg̃f[^̗sɕϊ
		List<List<TransitInfo>> columnEventList = getColumnEventList(contents);

		List<List<List<TransitInfo[]>>> resultTestCasesTransit = calcEventMetrix(
				rowEventList, columnEventList);
		
		List<List<String>> testCases = convertMatrixToTestCaseRow(resultTestCasesTransit);
		
		if(testCases != null && testCases.size() > 0){
			addTestCaseHeader(testCases);
		}
		
		
		return testCases;
	}
	
	private void addTestCaseHeader(List<List<String>> testCases){

		for (int i = 0; i < testCases.size(); i++) {
		
			testCases.get(i).add(0,IStateMachineTestConstants.TEST_CASE_TITLE + "_" + i);
			
		}
		
		List<String> titleRow = new ArrayList<String>();
		
		titleRow.add(0,IStateMachineTestConstants.TEST_CASE_TITLE);

		for(int i = 1; i < testCases.get(0).size(); i++){
			if(i % 2 != 0){
				titleRow.add(IStateMachineTestConstants.STATE);
			}else{
				titleRow.add(IStateMachineTestConstants.EVENT);
			}
		}
		testCases.add(0,titleRow);
	}
	
	protected List<List<List<TransitInfo[]>>> calcEventMetrix(
			List<List<TransitInfo>> rowEventList,
			List<List<TransitInfo>> columnEventList) {

		List<List<List<TransitInfo[]>>> resultTestCasesTransit = new ArrayList<List<List<TransitInfo[]>>>();

		// sZ
		for (int rowCount = 0; rowCount < rowEventList.size(); rowCount++) {
			// V\1s
			List<List<TransitInfo[]>> resultRow = new ArrayList<List<TransitInfo[]>>();

			for (int columnEventRowCount = 0; columnEventRowCount < columnEventList
					.size(); columnEventRowCount++) {

				List<TransitInfo[]> resultCell = new ArrayList<TransitInfo[]>();
				for (int columnCount = 0; columnCount < rowEventList.get(
						rowCount).size(); columnCount++) {
					
					resultCell.add(new TransitInfo[] { rowEventList.get(rowCount).get(columnCount), columnEventList.get(columnEventRowCount).get(columnCount)} );
				}
				resultRow.add(resultCell);
			}
			resultTestCasesTransit.add(resultRow);
		}
		return resultTestCasesTransit;
	}

	
	protected List<List<String>> convertMatrixToTestCaseRow(
			List<List<List<TransitInfo[]>>> resultTestCasesTransit) {
		
		List<List<String>> testCases = new ArrayList<List<String>>();

		// 	s[v
		for (List<List<TransitInfo[]>> rows : resultTestCasesTransit) {
			
			// Cell[v
			for (List<TransitInfo[]> cells : rows) {
				
				// eXgP[X̑gݍ킹
				for (TransitInfo[] transitInfos : cells) {

					List<String> testPath = new ArrayList<String>();
					
					// Cxg-Xe[g
					for (int testPathCount = 0; testPathCount < transitInfos.length; testPathCount++) {
						
						if(StringUtils.isEmpty(transitInfos[testPathCount].getTransitEvent()) || transitInfos[testPathCount].getTransitEvent().equals(IStateMachineTestConstants.FORBIDDEN)){
							testPath = null;
							break;
						}else{
							if(testPathCount == 0){
								testPath.add(transitInfos[testPathCount].getPreStateName());
							}else{
								if(!transitInfos[testPathCount-1].getPostStateName().equals(transitInfos[testPathCount].getPreStateName())){
									testPath = null;
									break;
								}
							}
							testPath.add(transitInfos[testPathCount].getTransitEvent());
							testPath.add(transitInfos[testPathCount].getPostStateName());
						}
					}

					if(testPath != null){
						testCases.add(testPath);
					}
				}
			}
		}
		return testCases;
	}


	// contentsCxg𒊏o
	protected List<List<TransitInfo>> getRowEventList(List<List<String>> contents) {

		List<List<TransitInfo>> rowEventList = new ArrayList<List<TransitInfo>>();

		for (int i = 1; i < contents.size(); i++) {
			List<TransitInfo> row = new ArrayList<TransitInfo>();

			for (int j = 1; j < contents.get(i).size(); j++) {

				TransitInfo transitInfo = new TransitInfo(rowTitleMap.get(i)
						.toString(), columnTitleMap.get(j).toString(),
						(String) contents.get(i).get(j));

				row.add(transitInfo);
			}
			rowEventList.add(row);
		}
		return rowEventList;
	}

	// ĴPf[^擾
	protected List<List<TransitInfo>> getColumnEventList(
			List<List<String>> contents) {

		List<List<TransitInfo>> columnBasedEventList = new ArrayList<List<TransitInfo>>();

		for (int columnNum = 1; columnNum < contents.get(0).size(); columnNum++) {

			List<TransitInfo> columnEventList = new ArrayList<TransitInfo>();

			for (int rowNum = 1; rowNum < contents.size(); rowNum++) {
				
				String rowState = rowTitleMap.get(rowNum).toString();
				String columnState = columnTitleMap.get(columnNum).toString();
				String transit = (String) contents.get(rowNum).get(columnNum);
				
				TransitInfo transitInfo = new TransitInfo(rowState,columnState,transit);

				columnEventList.add(transitInfo);
			}

			columnBasedEventList.add(columnEventList);
		}

		return columnBasedEventList;

	}

	// JCfbNXƃ^Cg̃}bv쐬
	private void setUpColumnTitleMap(List<String> titleRows) {

		this.columnTitleMap = new HashMap<Integer,String>();

		for (int i = 0; i < titleRows.size(); i++) {
			this.columnTitleMap.put( i,titleRows.get(i));
		}

	}

	// sCfbNXƍs^Cg̃}bv쐬
	private void setUpRowTitleMap(List<List<String>> contents) {

		this.rowTitleMap = new HashMap<Integer,String>();

		for (int i = 0; i < contents.size(); i++) {
			this.rowTitleMap.put(i,contents.get(i).get(0));
		}
	}
	
	protected void initSetUpTitleData(List<List<String>> contents){
		setUpColumnTitleMap(contents.get(0));
		setUpRowTitleMap(contents);
	}
}
